/******************************************************************************
/*   This file is part of the game Various Burst.
 *
 *   Various Burst is free software: you can redistribute it and/or modify it
 *   under the terms of the WTF Public License.
 *
 *   Virus Burst is distributed in the hope that players will find it enjoyable,
 *   emotive, educational even, but with NO WARRANTY of any kind. What happens
 *   to your Pandora, your dog or your sanity as a result of this is solely
 *   YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */
#include "common.h"
#include "burst.h"
#include "blenders.h"

#define  BURST_NUM_FRAMES  10

static int  burst_x;
static int  burst_y;
static int  burst_frame       = 0;
static BOOL burst_direction   = FALSE;

static BITMAP *burst_gfx[BURST_NUM_FRAMES];

/****************************************************************************/

void BURST_load(void)
{
   char buf[FILENAME_BUFFER_LENGTH];
   int index;

   for (index = 0; index < BURST_NUM_FRAMES; index++)
   {
      snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/burst/%02d.pcx", index + 1);

      if (exists(buf))
      {
         burst_gfx[index] = load_bitmap(buf, NULL);
      }
      else
      {
         burst_gfx[index] = create_bitmap(1, 1);
         DUH_WHERE_AM_I("missing asset: %s", buf);
      }
   }
}

/****************************************************************************/

void BURST_unload(void)
{
   int index;

   for (index = 0; index < BURST_NUM_FRAMES; index++)
   {
      destroy_bitmap(burst_gfx[index]);
   }
}

/****************************************************************************/

void BURST_draw(void)
{
   if (burst_frame < 1) return;

   set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 0, 0, 0, 64);

   if (!burst_direction)
   {
      draw_trans_sprite(double_buffer, burst_gfx[burst_frame - 1],
         burst_x - (burst_gfx[burst_frame - 1]->w / 2),
         burst_y - (burst_gfx[burst_frame - 1]->h / 2));
   }
   else
   {
      draw_trans_sprite(double_buffer, burst_gfx[BURST_NUM_FRAMES - burst_frame],
         burst_x - (burst_gfx[BURST_NUM_FRAMES - burst_frame]->w / 2),
         burst_y - (burst_gfx[BURST_NUM_FRAMES - burst_frame]->h / 2));
   }
}

/****************************************************************************/

void BURST_tick(void)
{
   if (burst_frame > 0)
      burst_frame--;
}

/****************************************************************************/

void BURST_reset_all(void)
{
   burst_frame = 0;
}

/****************************************************************************/

void BURST_spawn(int x, int y, BOOL in_or_out)
{
   burst_x = x;
   burst_y = y;
   burst_direction = in_or_out;
   burst_frame = BURST_NUM_FRAMES;
}
