/******************************************************************************
/*   This file is part of the game Various Burst.
 *
 *   Various Burst is free software: you can redistribute it and/or modify it
 *   under the terms of the WTF Public License.
 *
 *   Various Burst is distributed in the hope that players will find it enjoyable,
 *   emotive, educational even, but with NO WARRANTY of any kind. What happens
 *   to your Pandora, your dog or your sanity as a result of this is solely
 *   YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */

#include        <stdio.h>
#include        <stdint.h>
#include        <stdlib.h>
#include        <math.h>
#include        <allegro.h>
#include        <aldumb.h>

#ifndef         _WIN32
   #include    <unistd.h>
#endif

#ifdef          __OPENPANDORA__
    #include    <linux/fb.h>
    #include    <sys/ioctl.h>
    #include    <string.h>
    #include    <sys/mman.h>
#endif

#ifndef        __COMMON__
   #define     __COMMON__

   #define     DUH_WHERE_AM_I(...) {                                        \
                                  fprintf(stderr,"\n%s, %s(), line %i: ",   \
                                  __FILE__, __FUNCTION__, __LINE__);        \
                                  fprintf(stderr, __VA_ARGS__);             \
                              }

   #define     TO_RADIANS(theta)             (((theta * M_PI) / 180.0))
   #define     TO_DEGREES(theta)             ((theta * (180.0 / M_PI)))
   #define     ALLEGRODEGS_TO_RADIANS(theta) (((theta * M_PI) / 128.0))

   #define     FILENAME_BUFFER_LENGTH     2048

   #ifndef     FALSE
      #define  FALSE   0
   #endif

   #ifndef     TRUE
      #define  TRUE    ~FALSE
   #endif

   typedef  unsigned    BOOL;

   #define  DEFAULT_FONT_PATH    "./gfx/font/gamefont.bmp"
   #define  DISPLAY_WIDTH        400
   #define  DISPLAY_HEIGHT       240

   #define  TICKS_PER_SEC        60

   #define  DUMB_BUFSIZE         4096
   #define  DUMB_MIXRATE         44100
   #define  DUMB_FADE_MUL        0.95f
   #define  DUMB_FADE_MIN        0.005f
   #define  DUMB_VOL             0.92f
   #define  FONT_HEIGHT          8

   #define  STATE_EXIT_APP  9999
   #define  STATE_MENU      0
   #define  STATE_GAMEPLAY  1
   #define  NUM_GAMESTATES  2

   /*!
    * @brief A structure that holds what to do for any of the states the application
    * can be in.  A state is a self-contained area  of the application, like the main
    * menu, the high score screen, etc.
    */
   typedef struct
   {
      void    (*init)(void);
      void    (*load)(void);
      void    (*unload)(void);
      void    (*tick)(void);
      void    (*draw)(void);
   } GAMESTATE;

   extern   AL_DUH_PLAYER    *COMMON_duhplyr;
   extern   int              bgm_fade_done;
   extern   int              next_state;
   extern   BOOL             curr_state_done;
   extern   volatile int     time_for_logic;
   extern   BITMAP           *double_buffer;

   void     COMMON_setup(void);
   void     COMMON_shutdown(void);
   void     COMMON_flip_buffer(void);
   void     COMMON_show_fps(void);
   void     COMMON_fade_bgm(void);
   void     COMMON_set_bgm(int track);
   int      COMMON_get_bgm(void);

#endif
