#include "common.h"
#include "input.h"
#include "blenders.h"
#include "deadeffect.h"

#define  DEADEFFECT_ANIMCLOCK_START    64

static BITMAP  *deadeffect_buffer   = NULL;
static BITMAP  *deadeffect_gameover = NULL;
static SAMPLE  *deadeffect_endsound = NULL;

static int     deadeffect_animclock = 0;
static int     deadeffect_state     = DEADEFFECT_STATE_INACTIVE;

/******************************************************************************/

void DEADEFFECT_load(void)
{
   char buf[FILENAME_BUFFER_LENGTH];

   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/gameover/gameover.pcx");

   if (exists(buf))
   {
      deadeffect_gameover = load_bitmap(buf, NULL);
   }
   else
   {
      deadeffect_gameover = create_bitmap(1, 1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   snprintf(buf, FILENAME_BUFFER_LENGTH, "./snd/gameover/gameover.wav");

   if (exists(buf))
   {
      deadeffect_endsound = load_sample(buf);
   }
   else
   {
      deadeffect_endsound = create_sample(8,0,1,1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   deadeffect_buffer = create_bitmap(deadeffect_gameover->w, deadeffect_gameover->h);
}

/******************************************************************************/

void DEADEFFECT_tick(void)
{
   switch (deadeffect_state)
   {
      case DEADEFFECT_STATE_SLIDE_IN:
      {
         deadeffect_animclock--;
         if (deadeffect_animclock == 0)
         {
            deadeffect_state = DEADEFFECT_STATE_WAIT_USER;
         }
      }
      break;

      case DEADEFFECT_STATE_WAIT_USER:
      {
         if ((FIRE_B == BTN_PRESSED) || (FIRE_START == BTN_PRESSED))
         {
            deadeffect_state = DEADEFFECT_STATE_SLIDE_OUT;
         }
      }
      break;

      case DEADEFFECT_STATE_SLIDE_OUT:
      {
         deadeffect_animclock++;
         if (deadeffect_animclock == DEADEFFECT_ANIMCLOCK_START)
         {
            deadeffect_state = DEADEFFECT_STATE_DONE;
         }
      }
      break;
   }
}

/******************************************************************************/

void DEADEFFECT_draw(void)
{
   int sine_index;

   switch (deadeffect_state)
   {
      case DEADEFFECT_STATE_SLIDE_IN:
      {

         // darken game display
         set_blender_mode(NULL, (BLENDER_FUNC)BLEND_sub16, NULL, 0, 0, 0, 64 - (deadeffect_animclock));
         drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
         rectfill(double_buffer, 0, 0, DISPLAY_WIDTH, DISPLAY_HEIGHT, 0xFFFF);

         // sinewave logo effect into compositing buffer
         clear_bitmap(deadeffect_buffer);

         for (sine_index = 0; sine_index < deadeffect_buffer->h; sine_index++)
         {
            blit(deadeffect_gameover, deadeffect_buffer, 0, sine_index,
               sin((sine_index + deadeffect_animclock) / 3.0f) * (deadeffect_animclock / 3.0f), sine_index, deadeffect_buffer->w, 1);
         }

         // add compositing buffer to display
         set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 0, 0, 0, 64 - deadeffect_animclock);
         draw_trans_sprite(double_buffer, deadeffect_buffer, (DISPLAY_WIDTH - deadeffect_buffer->w) / 2, 32);

         // cleanup
         drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
      }
      break;

      case DEADEFFECT_STATE_WAIT_USER:
      {
         clear_bitmap(double_buffer);
         draw_sprite(double_buffer, deadeffect_buffer, (DISPLAY_WIDTH - deadeffect_buffer->w) / 2, 32);
      }
      break;

      case DEADEFFECT_STATE_SLIDE_OUT:
      {
         clear_bitmap(double_buffer);

         // sinewave logo effect into compositing buffer
         clear_bitmap(deadeffect_buffer);

         for (sine_index = 0; sine_index < deadeffect_buffer->h; sine_index++)
         {
            blit(deadeffect_gameover, deadeffect_buffer, 0, sine_index,
               sin((sine_index + deadeffect_animclock) / 4.0f) * (deadeffect_animclock / 3.0f), sine_index, deadeffect_buffer->w, 1);
         }

         draw_sprite(double_buffer, deadeffect_buffer, (DISPLAY_WIDTH - deadeffect_buffer->w) / 2, 32);

         // darken game display
         set_blender_mode(NULL, (BLENDER_FUNC)BLEND_sub16, NULL, 0, 0, 0, (deadeffect_animclock));
         drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
         rectfill(double_buffer, 0, 0, DISPLAY_WIDTH, DISPLAY_HEIGHT, 0xFFFF);
      }
      break;

   }
}

/******************************************************************************/

void DEADEFFECT_unload(void)
{
   destroy_bitmap(deadeffect_gameover);
   destroy_bitmap(deadeffect_buffer);
   destroy_sample(deadeffect_endsound);
}

/******************************************************************************/

void DEADEFFECT_start(void)
{
   if (deadeffect_state != DEADEFFECT_STATE_INACTIVE) return;
   deadeffect_state = DEADEFFECT_STATE_SLIDE_IN;
   deadeffect_animclock = DEADEFFECT_ANIMCLOCK_START;
   play_sample(deadeffect_endsound, 255, 128, 1000, 0);
}

/******************************************************************************/

unsigned DEADEFFECT_get_state(void)
{
   return deadeffect_state;
}

/******************************************************************************/

void DEADEFFECT_init(void)
{
   deadeffect_state = DEADEFFECT_STATE_INACTIVE;
}
