/******************************************************************************
/*   This file is part of the game Various Burst.
 *
 *   Various Burst is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Various Burst is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 *
 *    BUGS: walks the list about a zillion times when trying
 *    to add an entry; someone should do something about that...
 */

#include "common.h"
#include "highscore.h"
#include <stdlib.h>
#define  HIGHSCORE_FILE_PATH     "./vburst_highscore" //fixme
#define  HIGHSCORE_NUM_ENTRIES   5

typedef struct
{
   char        initials[4];
   unsigned    score;
} HIGHSCORE;

static BOOL       highscore_is_inited = FALSE;
static HIGHSCORE  highscore_table[HIGHSCORE_NUM_ENTRIES];

/******************************************************************************/

unsigned HIGHSCORE_compute_total(unsigned num_stages, unsigned time_survived, unsigned coins_gotten)
{
   return (num_stages * 10) + (coins_gotten * 5) + (time_survived);
}

/******************************************************************************/

void HIGHSCORE_flush(void)
{
   if (!highscore_is_inited) return;

   FILE *f = fopen(HIGHSCORE_FILE_PATH, "wb");
   int index;

   for (index = 0; index < HIGHSCORE_NUM_ENTRIES; index++)
   {
      fwrite(highscore_table[index].initials, 4, 1, f);
      fwrite(&highscore_table[index].score, 4, 1, f);
   }

   fclose(f);
}

/******************************************************************************/

void HIGHSCORE_init(void)
{
   if (highscore_is_inited) return;

   highscore_is_inited = TRUE;
   atexit(HIGHSCORE_flush);

   if (exists(HIGHSCORE_FILE_PATH))
   {
      FILE *f = fopen(HIGHSCORE_FILE_PATH, "rb");
      int index;

      for (index = 0; index < HIGHSCORE_NUM_ENTRIES; index++)
      {
         fread(highscore_table[index].initials, 4, 1, f);
         fread(&highscore_table[index].score, 4, 1, f);
         highscore_table[index].initials[3] = 0;
      }

      fclose(f);
   }
   else
   {
      // no actual high scores yet - create some
      highscore_table[0].score       = 1000;
      highscore_table[0].initials[0] = 'R';
      highscore_table[0].initials[1] = 'E';
      highscore_table[0].initials[2] = 'I';
      highscore_table[0].initials[3] = 0;

      highscore_table[1].score       = 900;
      highscore_table[1].initials[0] = 'I';
      highscore_table[1].initials[1] = 'S';
      highscore_table[1].initials[2] = '.';
      highscore_table[1].initials[3] = 0;

      highscore_table[2].score       = 800;
      highscore_table[2].initials[0] = '.';
      highscore_table[2].initials[1] = 'M';
      highscore_table[2].initials[2] = 'Y';
      highscore_table[2].initials[3] = 0;

      highscore_table[3].score       = 700;
      highscore_table[3].initials[0] = 'W';
      highscore_table[3].initials[1] = 'A';
      highscore_table[3].initials[2] = 'I';
      highscore_table[3].initials[3] = 0;

      highscore_table[4].score       = 600;
      highscore_table[4].initials[0] = 'F';
      highscore_table[4].initials[1] = 'U';
      highscore_table[4].initials[2] = '.';
      highscore_table[4].initials[3] = 0;
   }
}

/******************************************************************************/

void HIGHSCORE_append(unsigned score, const char *initials)
{
   HIGHSCORE_init();
   int index;

   for (index = HIGHSCORE_NUM_ENTRIES - 1; index >= 0; index--)
   {
      if (score > highscore_table[index].score)
      {
         if (index > (0))
         {
            highscore_table[index].score       = highscore_table[index - 1].score;
            highscore_table[index].initials[0] = highscore_table[index - 1].initials[0];
            highscore_table[index].initials[1] = highscore_table[index - 1].initials[1];
            highscore_table[index].initials[2] = highscore_table[index - 1].initials[2];
            highscore_table[index].initials[3] = 0;
         }
      }

      if ((index == 0) || (highscore_table[index - 1].score > score))
      {
         highscore_table[index].score       = score;
         highscore_table[index].initials[0] = initials[0];
         highscore_table[index].initials[1] = initials[1];
         highscore_table[index].initials[2] = initials[2];
         highscore_table[index].initials[3] = 0;
         return;
      }
   }
}

/******************************************************************************/

BOOL HIGHSCORE_check_ranked(unsigned score)
{
   HIGHSCORE_init();
   int index;

   for (index = 0; index < HIGHSCORE_NUM_ENTRIES; index++)
   {
      if (score > highscore_table[index].score)
      {
         return TRUE;
      }
   }

   return FALSE;
}

/******************************************************************************/

void HIGHSCORE_display(BITMAP *dest)
{
   HIGHSCORE_init();

   int index = 0;

   textprintf_centre_ex(dest, font, DISPLAY_WIDTH / 2, 140, -1, -1, "best bursters");

   for (index = 0; index < HIGHSCORE_NUM_ENTRIES; index++)
   {
      textprintf_centre_ex(dest, font, DISPLAY_WIDTH / 2, 140 + ((index+2) * 8), -1, -1, "%d. %s %08d",index + 1,
         highscore_table[index].initials, highscore_table[index].score);
   }
}
