/******************************************************************************
/*   This file is part of the game Various Burst.
 *
 *   Various Burst is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Various Burst is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */
#include <allegro.h>
#include "input.h"
#include "common.h"

#ifdef __OPENPANDORA__
    #include    <linux/input.h>
    #include    <sys/ioctl.h>
    int         fd_dpad;
//    int         fd_left_nub; // analogue is not used in this game
//    int         fd_right_nub;
#endif

/******************************************************************************/

unsigned JOY_UP;
unsigned JOY_DOWN;
unsigned JOY_LEFT;
unsigned JOY_RIGHT;

unsigned FIRE_A;
unsigned FIRE_B;
unsigned FIRE_X;
unsigned FIRE_Y;
unsigned FIRE_L;
unsigned FIRE_R;

unsigned FIRE_START;
unsigned FIRE_SELECT;

/******************************************************************************/

int INPUT_open_input_device(const char *device)
{
#ifdef __OPENPANDORA__
    int     index;
    char    filename[1024];
    char    device_name[1024];
    int     fd;
    int     result;

    for(index = 0; index < 256; index++)
    {
        snprintf(filename,sizeof(filename),"/dev/input/event%d",index);

        fd = open(filename, O_RDONLY);

        if(fd > 0)
        {
            ioctl(fd, EVIOCGNAME(sizeof(device_name)), device_name);

            result = strcmp(device_name, device);

            if(result == 0)
            {
                /* device names matched, this is what we want   */
                return fd;
            }

            /* not the right device.                                    */
            close(fd);
        }
    }

#endif

    /* no matching device found */
    return -1;
}

/******************************************************************************/

void INPUT_open_devices(void)
{
#ifdef __OPENPANDORA__
    fd_dpad         = INPUT_open_input_device("gpio-keys");
//    fd_left_nub     = INPUT_open_input_device("nub0");
//    fd_right_nub    = INPUT_open_input_device("nub1");
#endif
    return;
}

/******************************************************************************/

void INPUT_close_devices(void)
{
#ifdef __OPENPANDORA__
    close(fd_dpad);
//    close(fd_left_nub);
//    close(fd_right_nub);

#endif
    return;
}

/******************************************************************************/

void INPUT_reset(void)
{
    JOY_UP      = 0;
    JOY_DOWN    = 0;
    JOY_LEFT    = 0;
    JOY_RIGHT   = 0;

    FIRE_A      = 0;
    FIRE_B      = 0;
    FIRE_X      = 0;
    FIRE_Y      = 0;
    FIRE_L      = 0;
    FIRE_R      = 0;

    FIRE_START  = 0;
    FIRE_SELECT = 0;
}

/******************************************************************************/

void INPUT_poll(void)
{
 #ifdef __OPENPANDORA__
    static unsigned char    buttons[KEY_MAX/8 + 1];

    memset(buttons, 0, sizeof(buttons));

    /* dpad and fire buttons */
    ioctl(fd_dpad, EVIOCGKEY(sizeof(buttons)), buttons);

    if(buttons[KEY_UP         / 8] & (1 << (KEY_UP         % 8)))
    {
        if(JOY_UP) JOY_UP = BTN_HELD;
        else JOY_UP = BTN_PRESSED;
    }
    else
    {
        JOY_UP = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_DOWN / 8] & (1 << (KEY_DOWN % 8)))
    {
        if(JOY_DOWN) JOY_DOWN = BTN_HELD;
        else JOY_DOWN = BTN_PRESSED;
    }
    else
    {
        JOY_DOWN = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_LEFT / 8] & (1 << (KEY_LEFT % 8)))
    {
        if(JOY_LEFT) JOY_LEFT = BTN_HELD;
        else JOY_LEFT = BTN_PRESSED;
    }
    else
    {
        JOY_LEFT = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_RIGHT / 8] & (1 << (KEY_RIGHT % 8)))
    {
        if(JOY_RIGHT) JOY_RIGHT = BTN_HELD;
        else JOY_RIGHT = BTN_PRESSED;
    }
    else
    {
        JOY_RIGHT = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_LEFTALT / 8] & (1 << (KEY_LEFTALT % 8)))
    {
        if(FIRE_START) FIRE_START = BTN_HELD;
        else FIRE_START = BTN_PRESSED;
    }
    else
    {
        FIRE_START = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_LEFTCTRL   / 8] & (1 << (KEY_LEFTCTRL   % 8)))
    {
        if(FIRE_SELECT) FIRE_SELECT = BTN_HELD;
        else FIRE_SELECT = BTN_PRESSED;
    }
    else
    {
        FIRE_SELECT = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_HOME / 8] & (1 << (KEY_HOME % 8)))
    {
        if(FIRE_A) FIRE_A = BTN_HELD;
        else FIRE_A = BTN_PRESSED;
    }
    else
    {
        FIRE_A = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_PAGEDOWN / 8] & (1 << (KEY_PAGEDOWN % 8)))
    {
        if(FIRE_X) FIRE_X = BTN_HELD;
        else FIRE_X = BTN_PRESSED;
    }
    else
    {
        FIRE_X = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_PAGEUP / 8] & (1 << (KEY_PAGEUP % 8)))
    {
        if(FIRE_Y) FIRE_Y = BTN_HELD;
        else FIRE_Y = BTN_PRESSED;
    }
    else
    {
        FIRE_Y = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_END / 8] & (1 << (KEY_END % 8)))
    {
        if(FIRE_B) FIRE_B = BTN_HELD;
        else FIRE_B = BTN_PRESSED;
    }
    else
    {
        FIRE_B = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_RIGHTSHIFT / 8] & (1 << (KEY_RIGHTSHIFT % 8)))
    {
        if(FIRE_L) FIRE_L = BTN_HELD;
        else FIRE_L = BTN_PRESSED;
    }
    else
    {
        FIRE_L = BTN_INACTIVE;
    }

    /////////////////////////////

    if(buttons[KEY_RIGHTCTRL / 8] & (1 << (KEY_RIGHTCTRL % 8)))
    {
        if(FIRE_R) FIRE_R = BTN_HELD;
        else FIRE_R = BTN_PRESSED;
    }
    else
    {
        FIRE_R = BTN_INACTIVE;
    }

#else
    poll_keyboard();

    if(key[KEY_UP])
        if(JOY_UP)
            JOY_UP = BTN_HELD;
        else
            JOY_UP = BTN_PRESSED;
    else
        JOY_UP = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_DOWN])
        if(JOY_DOWN)
            JOY_DOWN = BTN_HELD;
        else
            JOY_DOWN = BTN_PRESSED;
    else
        JOY_DOWN = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_LEFT])
        if(JOY_LEFT)
            JOY_LEFT = BTN_HELD;
        else
            JOY_LEFT = BTN_PRESSED;
    else
        JOY_LEFT = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_RIGHT])
        if(JOY_RIGHT)
            JOY_RIGHT = BTN_HELD;
        else
            JOY_RIGHT = BTN_PRESSED;
    else
        JOY_RIGHT = BTN_INACTIVE;

    //------------------------------

    if((key[KEY_ALT]) || (key[KEY_ENTER]))
        if(FIRE_START)
            FIRE_START = BTN_HELD;
        else
            FIRE_START = BTN_PRESSED;
    else
        FIRE_START = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_LCONTROL])
        if(FIRE_SELECT)
            FIRE_SELECT = BTN_HELD;
        else
            FIRE_SELECT = BTN_PRESSED;
    else
        FIRE_SELECT = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_PGDN] | key[KEY_Z])
        if(FIRE_X)
            FIRE_X = BTN_HELD;
        else
            FIRE_X = BTN_PRESSED;
    else
        FIRE_X = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_PGUP] | key[KEY_X])
        if(FIRE_Y)
            FIRE_Y = BTN_HELD;
        else
            FIRE_Y = BTN_PRESSED;
    else
        FIRE_Y = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_HOME] | key[KEY_C])
        if(FIRE_A)
            FIRE_A = BTN_HELD;
        else
            FIRE_A = BTN_PRESSED;
    else
        FIRE_A = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_END] | key[KEY_V])
        if(FIRE_B)
            FIRE_B = BTN_HELD;
        else
            FIRE_B = BTN_PRESSED;
    else
        FIRE_B = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_RCONTROL])
        if(FIRE_R)
            FIRE_R = BTN_HELD;
        else
            FIRE_R = BTN_PRESSED;
    else
        FIRE_R = BTN_INACTIVE;

    //------------------------------

    if(key[KEY_RSHIFT])
        if(FIRE_L)
            FIRE_L = BTN_HELD;
        else
            FIRE_L = BTN_PRESSED;
    else
        FIRE_L = BTN_INACTIVE;
#endif

}
