/******************************************************************************
/*   This file is part of the game Virus Burst.
 *
 *   Illuminous is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Illuminous is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */


#include "common.h"
#include "input.h"
#include "gameplay.h"
#include "mainmenu.h"
#include "splash.h"

GAMESTATE gamestates[NUM_GAMESTATES];

int main(void)
{
   int curr_state = STATE_MENU;

   // set up our game states...
   gamestates[STATE_MENU].init   = MAINMENU_init;
   gamestates[STATE_MENU].tick   = MAINMENU_tick;
   gamestates[STATE_MENU].load   = MAINMENU_load;
   gamestates[STATE_MENU].unload = MAINMENU_unload;
   gamestates[STATE_MENU].draw   = MAINMENU_draw;

   gamestates[STATE_GAMEPLAY].init   = GAMEPLAY_init;
   gamestates[STATE_GAMEPLAY].tick   = GAMEPLAY_tick;
   gamestates[STATE_GAMEPLAY].load   = GAMEPLAY_load;
   gamestates[STATE_GAMEPLAY].unload = GAMEPLAY_unload;
   gamestates[STATE_GAMEPLAY].draw   = GAMEPLAY_draw;

   next_state = STATE_MENU;
                                                    
   COMMON_setup();  
   INPUT_open_devices();                                                  
   do_splash();
   curr_state_done = FALSE;

   // Main loop
   while (next_state != STATE_EXIT_APP)
   {
      curr_state = next_state;
      curr_state_done = FALSE;

      gamestates[curr_state].load();
      gamestates[curr_state].init();

      while (curr_state_done == FALSE)
      {
         if (time_for_logic)
         {
            INPUT_poll();
            gamestates[curr_state].tick();
            gamestates[curr_state].draw();
            COMMON_flip_buffer();
            time_for_logic = 0;
         }
      }

      gamestates[curr_state].unload();
   }

   INPUT_close_devices();                                                  
   COMMON_shutdown();
}
END_OF_MAIN()
