/******************************************************************************
/*   This file is part of the game Virus Burst.
 *
 *   Illuminous is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Illuminous is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */


#include "common.h"
#include "blenders.h"
#include "input.h"
#include "backdrop.h"
#include "highscore.h"

static unsigned   mainmenu_logotype_scroll_val     =  0;
static BITMAP     *mainmenu_logotype_mask          =  NULL;
static BITMAP     *mainmenu_logotype_scroll        =  NULL;
static BITMAP     *mainmenu_logotype_overlay       =  NULL;
static BITMAP     *mainmenu_logotype_buffer        =  NULL;

static int        mainmenu_fadein_clock            =  0;

static int        mainmenu_fadeout_clock           =  0;

static BOOL       mainmenu_option_chosen           =  FALSE;

/******************************************************************************/

void MAINMENU_load(void)
{
   char buf[FILENAME_BUFFER_LENGTH];

   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/mainmenu/logo-mask.pcx");

   if (exists(buf))
   {
      mainmenu_logotype_mask = load_bitmap(buf, NULL);
   }
   else
   {
      mainmenu_logotype_mask = create_bitmap(1, 1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/mainmenu/logo-scroll.pcx");

   if (exists(buf))
   {
      mainmenu_logotype_scroll = load_bitmap(buf, NULL);
   }
   else
   {
      mainmenu_logotype_scroll = create_bitmap(1, 1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/mainmenu/logo-overlay.pcx");

   if (exists(buf))
   {
      mainmenu_logotype_overlay = load_bitmap(buf, NULL);
   }
   else
   {
      mainmenu_logotype_overlay = create_bitmap(1, 1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   mainmenu_logotype_buffer = create_bitmap(mainmenu_logotype_overlay->w, mainmenu_logotype_overlay->h);
}

/******************************************************************************/

void MAINMENU_unload(void)
{
   destroy_bitmap(mainmenu_logotype_buffer);
   destroy_bitmap(mainmenu_logotype_mask);
   destroy_bitmap(mainmenu_logotype_scroll);
   destroy_bitmap(mainmenu_logotype_overlay);
}

/******************************************************************************/

void MAINMENU_init(void)
{
   COMMON_set_bgm(99);
   mainmenu_logotype_scroll_val = 0;
   mainmenu_option_chosen = FALSE;
   mainmenu_fadeout_clock = 0;
   mainmenu_fadein_clock  = 63;
}

/******************************************************************************/

static const int BLINK_COUNTER_MAX  = 224;

void MAINMENU_draw(void)
{
   clear_bitmap(double_buffer);

   static int blink_counter = 0;

   blink_counter++;

   // logo scrolling background
   blit(mainmenu_logotype_scroll, mainmenu_logotype_buffer, 0, 0, mainmenu_logotype_scroll_val, mainmenu_logotype_scroll_val,
      mainmenu_logotype_scroll->w, mainmenu_logotype_scroll->h);

   blit(mainmenu_logotype_scroll, mainmenu_logotype_buffer, 0, 0,
      mainmenu_logotype_scroll_val - mainmenu_logotype_scroll->w,
      mainmenu_logotype_scroll_val,
      mainmenu_logotype_scroll->w, mainmenu_logotype_scroll->h);

   blit(mainmenu_logotype_scroll, mainmenu_logotype_buffer, 0, 0,
      mainmenu_logotype_scroll_val - mainmenu_logotype_scroll->w,
      mainmenu_logotype_scroll_val - mainmenu_logotype_scroll->h,
      mainmenu_logotype_scroll->w, mainmenu_logotype_scroll->h);

   blit(mainmenu_logotype_scroll, mainmenu_logotype_buffer, 0, 0,
      mainmenu_logotype_scroll_val,
      mainmenu_logotype_scroll_val - mainmenu_logotype_scroll->h,
      mainmenu_logotype_scroll->w, mainmenu_logotype_scroll->h);

   // logo mask
   draw_sprite(mainmenu_logotype_buffer, mainmenu_logotype_mask, 0, 0);

   // logo glowing overlay
   set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 0, 0, 0,
      31 + (sin(blink_counter / 47.5f) * 32));
   draw_trans_sprite(mainmenu_logotype_buffer, mainmenu_logotype_overlay, 0, 0);

   // compositing done; copy compositing buffer to framebuffer
   set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 0, 0, 0, 64);
   draw_trans_sprite(double_buffer, mainmenu_logotype_buffer,
      (DISPLAY_WIDTH - mainmenu_logotype_buffer->w) / 2,
      16);


   HIGHSCORE_display(double_buffer);

   if ((blink_counter % BLINK_COUNTER_MAX) < (BLINK_COUNTER_MAX / 2))
   {
      textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 216, -1, -1, "PRESS (start) TO PLAY");
   }
   else
   {
      textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 216, -1, -1, "PRESS (select) TO QUIT");
   }

   if (mainmenu_fadeout_clock > 0)
   {
      set_blender_mode(NULL, (BLENDER_FUNC)BLEND_sub16, NULL, 0, 0, 0, (mainmenu_fadeout_clock));
      drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
      rectfill(double_buffer, 0, 0, DISPLAY_WIDTH, DISPLAY_HEIGHT, 0xFFFF);
   }

   if (mainmenu_fadein_clock > 0)
   {
      set_blender_mode(NULL, (BLENDER_FUNC)BLEND_sub16, NULL, 0, 0, 0, (mainmenu_fadein_clock));
      drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
      rectfill(double_buffer, 0, 0, DISPLAY_WIDTH, DISPLAY_HEIGHT, 0xFFFF);
   }
}

/******************************************************************************/

void MAINMENU_tick(void)
{
   // logotype animation
   mainmenu_logotype_scroll_val++;

   if (mainmenu_fadein_clock > 0)
      mainmenu_fadein_clock -= 2;

   if (mainmenu_logotype_scroll_val >= mainmenu_logotype_scroll->h)
      mainmenu_logotype_scroll_val = 0;

   if (!mainmenu_option_chosen)
   {
      if (FIRE_START == BTN_PRESSED)
      {
         mainmenu_option_chosen  = TRUE;
         next_state              = STATE_GAMEPLAY;
         COMMON_fade_bgm();
      }
      else
      if (FIRE_SELECT == BTN_PRESSED)
      {
         mainmenu_option_chosen  = TRUE;
         next_state              = STATE_EXIT_APP;
         COMMON_fade_bgm();
      }
   }
   else
   {
      mainmenu_fadeout_clock++;

      if (mainmenu_fadeout_clock >= 64)
      {
         curr_state_done = TRUE;
      }
   }
}


