/******************************************************************************
/*   This file is part of the game Virus Burst.
 *
 *   Illuminous is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Illuminous is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */


#include "common.h"
#include "nameentry.h"
#include "input.h"
#include "blenders.h"

static char name_initials[4];
static int  name_char_index      = 0;
static int  name_initial_index   = 0;
static BOOL name_done            = FALSE;
static BOOL name_started         = FALSE;
static BITMAP *name_logo         = NULL;

void NAME_load(void)
{
   char buf[FILENAME_BUFFER_LENGTH];

   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/highscore/logo.pcx");

   if (exists(buf))
   {
      name_logo = load_bitmap(buf, NULL);
   }
   else
   {
      name_logo = create_bitmap(1, 1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }
}

/******************************************************************************/

void NAME_unload(void)
{
   destroy_bitmap(name_logo);
}

/******************************************************************************/

void NAME_init(void)
{
   name_initials[0] = ' ';
   name_initials[1] = ' ';
   name_initials[2] = ' ';
   name_initials[3] = 0;

   name_char_index      = 0;
   name_initial_index   = 0;
   name_done            = FALSE;
   name_started         = FALSE;
}

/******************************************************************************/

void NAME_start(void)
{
   name_started = TRUE;
}

/******************************************************************************/

BOOL NAME_check_done(void)
{
   return (name_started && name_done);
}

/******************************************************************************/

void NAME_tick(void)
{
   if (!name_started) return;

   // choose letter or number with stick/dpad
   if (JOY_UP == BTN_PRESSED)
   {
      name_char_index++;
   }

   if (JOY_DOWN == BTN_PRESSED)
   {
      name_char_index--;
   }

   // range checking on letter/number choice
   if (name_char_index < 0) name_char_index = 35;
   if (name_char_index > 35) name_char_index = 0;

   // place the character in the string
   if (name_char_index < 26)
   {
      name_initials[name_initial_index] = 'A' + name_char_index;
   }
   else
   {
      name_initials[name_initial_index] = '0' + name_char_index - 26;
   }

   // move cursor
   if (FIRE_B == BTN_PRESSED)
   {
      name_initial_index++;
   }

   if (name_initial_index >= 3)
   {
      name_done = TRUE;
   }
}

/******************************************************************************/

void NAME_draw(void)
{
   if (!name_started) return;

   set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 32, 32, 32, 64);
   draw_trans_sprite(double_buffer, name_logo, (DISPLAY_WIDTH - name_logo->w) / 2, 16);

   textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 128, -1, -1, "CONGRATULATIONS!");
   textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 136, -1, -1, "YOU GOT A HIGH SCORE.");
   textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 144, -1, -1, "please enter your initials:");

   switch (name_initial_index)
   {
      case 0:
         textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 152, -1, -1, "]  ");
         textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 168, -1, -1, "[  ");
      break;

      case 1:
         textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 152, -1, -1, " ] ");
         textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 168, -1, -1, " [ ");
      break;

      case 2:
         textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 152, -1, -1, "  ]");
         textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 168, -1, -1, "  [");
      break;
   }

   textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 160, -1, -1, "%s", name_initials);

   textprintf_centre_ex(double_buffer, font, DISPLAY_WIDTH / 2, 192, -1, -1, "PRESS (B) TO ADVANCE");
}

/******************************************************************************/

char *NAME_fetch(void)
{
   return name_initials;
}
