/******************************************************************************
/*   This file is part of the game Virus Burst.
 *
 *   Illuminous is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Illuminous is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */

#include "common.h"
#include "pickups.h"
#include "blenders.h"
#include "burst.h"

#define  PICKUP_MIN_DIST_FROM_EDGE  32
#define  PICKUP_RADIUS              15.5
#define  PICKUP_TYPE_COIN           1
#define  PICKUP_TYPE_EXIT           2

static BITMAP *pickups_coin_gfx = NULL;
static BITMAP *pickups_exit_gfx = NULL;

static int     pickups_curr_x;
static int     pickups_curr_y;
static int     pickups_curr_type;

static void   (*pickups_exit_callback)() = NULL;
static void   (*pickups_coin_callback)() = NULL;

static SAMPLE *pickups_get_snd = NULL;

/******************************************************************************/

void PICKUPS_load(void)
{
   char buf[FILENAME_BUFFER_LENGTH];

   // chit
   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/pickups/0.pcx");

   if (exists(buf))
   {
      pickups_coin_gfx = load_bitmap(buf, NULL);
   }
   else
   {
      pickups_coin_gfx = create_bitmap(1,1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   // exit
   snprintf(buf, FILENAME_BUFFER_LENGTH, "./gfx/pickups/1.pcx");

   if (exists(buf))
   {
      pickups_exit_gfx = load_bitmap(buf, NULL);
   }
   else
   {
      pickups_exit_gfx = create_bitmap(1,1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }

   // get sound
   snprintf(buf, FILENAME_BUFFER_LENGTH, "./snd/coin/get.wav");

   if (exists(buf))
   {
      pickups_get_snd = load_sample(buf);
   }
   else
   {
      pickups_get_snd = create_sample(8, 1, 22050, 1);
      DUH_WHERE_AM_I("missing asset: %s", buf);
   }
}

/******************************************************************************/

void PICKUPS_unload(void)
{
   destroy_bitmap(pickups_coin_gfx);
   destroy_bitmap(pickups_exit_gfx);
   pickups_exit_callback = NULL;
   pickups_coin_callback = NULL;
   destroy_sample(pickups_get_snd);
}

/******************************************************************************/

void PICKUPS_spawn_coin(void)
{
   pickups_curr_type = PICKUP_TYPE_COIN;
   pickups_curr_x = PICKUP_MIN_DIST_FROM_EDGE + (rand() % (DISPLAY_WIDTH  - (2 * PICKUP_MIN_DIST_FROM_EDGE)));
   pickups_curr_y = PICKUP_MIN_DIST_FROM_EDGE + (rand() % (DISPLAY_HEIGHT - (2 * PICKUP_MIN_DIST_FROM_EDGE)));
   BURST_spawn(pickups_curr_x, pickups_curr_y, TRUE);
}

/******************************************************************************/

void PICKUPS_remove(void)
{
   // push far enough outside display that player can't reach
   pickups_curr_x = -1 - PICKUP_RADIUS;
   pickups_curr_y = -1 - PICKUP_RADIUS;
}

/******************************************************************************/

void PICKUPS_set_coin_hit_callback(void (*callback)(void))
{
   pickups_coin_callback = callback;
}

/******************************************************************************/

void PICKUPS_set_exit_hit_callback(void (*callback)(void))
{
   pickups_exit_callback = callback;
}

/******************************************************************************/

void PICKUPS_spawn_exit(void)
{
   pickups_curr_type = PICKUP_TYPE_EXIT;
   pickups_curr_x    = DISPLAY_WIDTH  / 2;
   pickups_curr_y    = DISPLAY_HEIGHT / 2;
}

/******************************************************************************/

void PICKUPS_tick(float hero_x, float hero_y, float hero_radius)
{
   float dist = hypot(pickups_curr_x - hero_x, pickups_curr_y - hero_y);

   if (dist <= (hero_radius + pickups_curr_type))
   {
      if (pickups_curr_type != PICKUP_TYPE_EXIT)
         BURST_spawn(pickups_curr_x, pickups_curr_y, FALSE);

      PICKUPS_remove();

      switch (pickups_curr_type)
      {
         case PICKUP_TYPE_COIN:
            if (pickups_coin_callback != NULL)
               pickups_coin_callback();
            play_sample(pickups_get_snd, 255, 128, 1000, 0);

         break;

         case PICKUP_TYPE_EXIT:
            if (pickups_exit_callback != NULL)
               pickups_exit_callback();
         break;

      }
   }
}

/******************************************************************************/

void PICKUPS_draw(void)
{
   if (pickups_curr_x < 0) return;

   set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 0, 0, 0, 64);

   switch (pickups_curr_type)
   {
      case PICKUP_TYPE_COIN:
         draw_trans_sprite(double_buffer, pickups_coin_gfx, pickups_curr_x - (pickups_coin_gfx->w /2),
            pickups_curr_y - (pickups_coin_gfx->w /2));
      break;

      case PICKUP_TYPE_EXIT:
         draw_trans_sprite(double_buffer, pickups_exit_gfx, pickups_curr_x - (pickups_exit_gfx->w /2),
            pickups_curr_y - (pickups_coin_gfx->h /2));
      break;

   }
}


