/******************************************************************************
/*   This file is part of the game Virus Burst.
 *
 *   Illuminous is free software: you can redistribute it and/or modify
 *   it under the terms of the WTF Public License.
 *
 *   Illuminous is distributed in the hope that players will find it
 *   enjoyable, emotive, educational even, but with NO WARRANTY of any
 *   kind.  What happens to your computer, your dog or your sanity as
 *   a result of this is solely YOUR responsibility.
 *
 *   You should have received a copy of the WTF Public License
 *   along with the game (please see the file COPYING).  If not,
 *   please refer to <http://www.wtfpl.net/about/>.
 */

#include "common.h"
#include "blurrer.h"
#include "warpeffect.h"
#include "blenders.h"

static BITMAP *warpeffect_buffer    = NULL;

static int  warpeffect_logic_count  = 0;
static BOOL warpeffect_enabled      = FALSE;

/******************************************************************************/

BOOL WARPEFFECT_check_enabled(void)
{
   return warpeffect_enabled;
}

/******************************************************************************/

void WARPEFFECT_load(void)
{
   warpeffect_buffer = create_bitmap(DISPLAY_WIDTH, DISPLAY_HEIGHT);
}

/******************************************************************************/

void WARPEFFECT_unload(void)
{
   destroy_bitmap(warpeffect_buffer);
}

/******************************************************************************/

void WARPEFFECT_set(BOOL enable)
{
   warpeffect_enabled = enable;
   if (warpeffect_enabled) warpeffect_logic_count = 12;
}

/******************************************************************************/

void WARPEFFECT_grab(void)
{
   if (warpeffect_logic_count == 0) return;

   if ((warpeffect_enabled == TRUE)  && (warpeffect_logic_count < 48)) warpeffect_logic_count++;
   if ((warpeffect_enabled == FALSE) && (warpeffect_logic_count > 0 )) warpeffect_logic_count--;

   pivot_scaled_sprite(warpeffect_buffer, double_buffer,
      DISPLAY_WIDTH / 2, DISPLAY_HEIGHT / 2,
      DISPLAY_WIDTH / 2, DISPLAY_HEIGHT / 2,
      ftofix(warpeffect_logic_count/8.0f), ftofix(1.0 + (warpeffect_logic_count / 160.0f)));
   BLUR_apply(warpeffect_buffer);
}

/******************************************************************************/

void WARPEFFECT_draw(void)
{
   if (warpeffect_logic_count == 0) return;

   set_blender_mode(NULL, (BLENDER_FUNC)BLEND_add16, NULL, 0, 0, 0, warpeffect_logic_count);
   draw_trans_sprite(double_buffer, warpeffect_buffer, 0, 0);
}
